////////////////////////////////////////////////////////////////////// 
// merlin/TreeManager.cpp 
// (c) 2000-2006 Goncalo Abecasis
// 
// This file is distributed as part of the LAMP source code package   
// and may not be redistributed in any form, without prior written    
// permission from the author. Permission is granted for you to       
// modify this file for your own personal use, but modified versions  
// must retain this copyright notice and must not be distributed.     
// 
// Permission is granted for you to use this file to compile LAMP.    
// 
// All computer programs have bugs. Use this file at your own risk.   
// 
// Friday September 01, 2006
// 
 
#include "TreeManager.h"
#include "TreeBasics.h"

void TreeManager::GetPosition(TreePosition & pos)
   {
#ifndef __HUGE_SWAP__
   pos.skeleton = ftell(skeleton);
   pos.leaves = ftell(leaves);
#else
   fgetpos(skeleton, &pos.skeleton);
   fgetpos(leaves, &pos.leaves);
#endif
   }

void TreeManager::SetPosition(const TreePosition & pos)
   {
#ifndef __HUGE_SWAP__
   fseek(skeleton, pos.skeleton, SEEK_SET);
   fseek(leaves, pos.leaves, SEEK_SET);
#else
   fsetpos(skeleton, &pos.skeleton);
   fsetpos(leaves, &pos.leaves);
#endif
   }

bool TreeManager::OpenFiles()
   {
   if (skeleton == NULL)
      {
      skeleton = tmpfile();
      leaves = tmpfile();

      return true;
      }

   return false;
   }

void TreeManager::CloseFiles()
   {
   if (skeleton != NULL)
      {
      fclose(skeleton);
      fclose(leaves);
      }

   skeleton = leaves = NULL;
   }

void TreeManager::Free()
   {
#ifndef __HUGE_SWAP__
   position.leaves = position.skeleton = 0;
#else
   if (skeleton == NULL) return;

   fseek(skeleton, 0, SEEK_SET);
   fseek(leaves, 0, SEEK_SET);

   fgetpos(skeleton, &position.skeleton);
   fgetpos(leaves, &position.leaves);
#endif
   }

double TreeManager::EstimateFileSize()
   {
   double      fileSize = 0;
   FILEINDEX_t intermediate;

   fseek(skeleton, 0, SEEK_END);
   fseek(leaves, 0, SEEK_END);

#ifndef __HUGE_SWAP__
   intermediate = ftell(skeleton);
#else
   fgetpos(skeleton, &intermediate);
#endif

   // If the following line generates a compilation error,
   // it can be safely commented out
   fileSize += intermediate;

#ifndef __HUGE_SWAP__
   intermediate = ftell(leaves);
#else
   fgetpos(leaves, &intermediate);
#endif

   // If the following line generates a compilation error,
   // it can be safely commented out
   fileSize += intermediate;

   return fileSize;
   }
 
